<?php
/*
Plugin Name: Fun with Photo Data
Plugin URI: http://www.wp-fun.co.uk/fun-with-photo-data/
Description: Adds a shortcode to display the EXIF and IPTC Data that WordPress 2.5 extracts.
Author: Andrew Rickmann
Version: 1.1.2
Author URI: http://www.wp-fun.co.uk
Generated At: www.wp-fun.co.uk;

Thanks to Joshua Lyman (http://www.podq.com/) for contributions to 1.1 (and 1.1.1) including:
The initial ideas to correct the basic data and use an output string
Adding the corrections and output string options into the code
The idea to automatically add the data to every post.
*/ 

/*  Copyright 2008  Andrew Rickmann  (email : PLUGIN AUTHOR EMAIL)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

if (!class_exists('fw_photo_data')) {
    class fw_photo_data	{

		/**
		* @var string   The name the options are saved under in the database.
		*/
		var $adminOptionsName = "fw_photo_data_options";
		
		/**
		* PHP 4 Compatible Constructor
		*/
		function fw_photo_data(){$this->__construct();}
		
		/**
		* PHP 5 Constructor
		*/		
		function __construct(){

		//Register the shortcode
		if ( function_exists( 'add_shortcode' ) ) {
			add_shortcode('photodata', array( &$this , 'photodata_shortcode_handler' ) );
		}
		
		//register the menu function
		add_action("admin_menu", array(&$this,"add_admin_pages"));
		
		//register the content altering bit
		add_filter("the_content", array(&$this,"add_photo_data"));
		
		//get the admin options.
		$this->adminOptions = $this->getAdminOptions();
		

		}
		
		/**
		 * 
		 * @return String Amended post content
		 * @param $content String The post Content
		 */
		function add_photo_data($content){
			global $post;
			
			//if auto category is not set then exit
			if ( !$this->adminOptions['auto_category'] ) { return $content;}
			
			//get the categories for this post
			$categories = wp_get_post_categories($post->ID);
			
			//match to the auto category (or not)
			if ( in_array( $this->adminOptions['auto_category'] , $categories )){
				//if we match return the content plus the shortcode
				return $content . "\n\n[photodata]";
			}
			
			
			return $content;
	
		}
		
		/**
		* Retrieves the options from the database.
		* @return array
		*/
		function getAdminOptions() {
		$adminOptions = array("auto_category" => false, "display_title" => '<h3>Photo Data</h3>', "output_string" => '');
		$savedOptions = get_option($this->adminOptionsName);
		if (!empty($savedOptions)) {
			foreach ($savedOptions as $key => $option) {
				$adminOptions[$key] = $option;
			}
		}
		update_option($this->adminOptionsName, $adminOptions);
		return $adminOptions;
		}
		
		/**
		* Saves the admin options to the database.
		*/
		function saveAdminOptions(){
			update_option($this->adminOptionsName, $this->adminOptions);
		}
		
		function add_admin_pages(){
				add_submenu_page('options-general.php', "Exif Data", "Exif Data", 10, "Exif Data", array(&$this,"output_sub_admin_page_0"));
		}
		
		/**
		* Outputs the HTML for the admin sub page.
		*/
		function output_sub_admin_page_0(){
			
			//deal with the data submission
			if ( isset($_POST['auto_category']) ) {
					
				if ( !wp_verify_nonce( $_POST['fun_with_photo_data_auto_cat'], plugin_basename(__FILE__) )) {
				  return;
				}
				
				$this->adminOptions['display_title'] = (get_magic_quotes_gpc()) ? stripslashes($_POST['display_title']) : $_POST['display_title'];
				$this->adminOptions['output_string'] = (get_magic_quotes_gpc()) ? stripslashes($_POST['output_string']) : $_POST['output_string'];
				
				if ( is_numeric($_POST['auto_category'])  ){
					$this->adminOptions['auto_category'] = $_POST['auto_category'];
					
					$this->saveAdminOptions();
				} else{
					$this->adminOptions['auto_category'] = false;
					$this->saveAdminOptions();
				}
				
			}
			
			
			
			
			?>

			<div class="wrap">
				<h2>Fun with Photo Data Options</h2>
				<p>If you want to include photo data automatically for the first image of each post in a specific category then select the category below.</p>
				<p>The output format field is optional, but may be used to override the default unordered list that is generated.</p>
				<form method="post">
				<table class="form-table">
					<tr>
						<th scope="row">Category</th>
						<td>
							<select name="auto_category">
								<option value="x">None</option>
								<?php
								$categories = get_categories(array("type" => 'post',
																	"orderby" => 'name',
																	"order" => 'ASC',
																	"hide_empty" => false));
								foreach ($categories as $cat) {
									$selected = ( $this->adminOptions['auto_category'] == $cat->cat_ID ) ? ' selected="selected"' : '';
								  	$option = '<option'.$selected.' value="'.$cat->cat_ID.'">';
									$option .= $cat->cat_name;
									$option .= '</option>';
									echo $option;
								  }
								?>
							</select>
						</td>
						
					</tr>
					<tr>
						<th>Title</th>
						<td><input name="display_title" value="<?php echo htmlspecialchars($this->adminOptions['display_title'], ENT_QUOTES); ?>" /></td>
					</tr>
					<tr>
					  	<th>Output Format </th>
				 	  	<td>    
					  		<input name="output_string" value="<?php echo htmlspecialchars($this->adminOptions['output_string'], ENT_QUOTES); ?>" size="80" />
				 	        <p>Variables Available: <em>?camera? | ?credit? | ?created_timestamp? | ?focal_length? | ?shutter_speed? | ?aperture?</em><br />
							Example: <em>&lt;p class="image_meta"&gt;Shot with a ?camera? by ?credit? on ?created_timestamp?. (?focal_length?, ?shutter_speed? @ ?aperture?)&lt;/p&gt;</em></p>
						</td>
				  </tr>
					<tr><td colspan="2"><input type="submit" class="button-secondary save" name="saveit" value="Save"/></td></tr>
				</table>
				<?php
				// Use nonce for verification
				echo '<input type="hidden" name="fun_with_photo_data_auto_cat" id="fun_with_photo_data_auto_cat" value="' . wp_create_nonce( plugin_basename(__FILE__) ) . '" />';
				?>
				</form>
			
			
			</div>
			<?php
		} 
		
		/**
		* photodata_shortcode_handler - produces and returns the content to replace the shortcode tag
		*
		* @param array $atts  An array of attributes passed from the shortcode
		* @param string $content   If the shortcode wraps round some html, this will be passed.
		*/
		function photodata_shortcode_handler( $atts , $content = null) {
			//add the attributes you want to accept to the array below
			$attributes = shortcode_atts(array(
		      'id' => 'none',
		      'url' => '',
			  'output_string' => '',
			  'fail' => 'No data is available for this image.'
		      // ...etc
			), $atts);
		
			$return_value = $attributes['fail'];
		
			if ( $attributes['id'] == 'none' && !empty($attributes['url'])) {
				//get the attachment ID using the URL
				//the quickest and easiest way to get this is to go direct to the database
				
				$attachments = array_values(get_children("post_type=attachment&post_mime_type=image"));

				foreach($attachments as $attachment){
					
					if ( $attachment->guid == $attributes['url'] ) {
						$attributes['id'] = $attachment->ID;
						break;
					}
				}
			}
			
			//if no attachment data is added then get the first image attached to the post
			//this is necessary for the auto attachment option.
			if ( $attributes['id'] == 'none' && empty($attributes['url'])) {
				global $post;
				
				$attachments = array_values(get_children("post_parent=".$post->ID."&post_type=attachment&post_mime_type=image"));

				//just use the first one whatever it is.
				$attributes['id'] = $attachments[0]->ID;
			
			}
			
			//Check to see if there is already a user-defined output string to use
			if ( !empty($this->adminOptions['output_string']) ) { $attributes['output_string'] = $this->adminOptions['output_string']; }
			
			if ( is_numeric( $attributes['id'] ) ) {
				//make sure we are dealing with an image
				if ( wp_attachment_is_image( $attributes['id'] ) ) {
					if ( $meta_data = wp_get_attachment_metadata( $attributes['id'] ) ) {
						//make sure that there is an array to check
						if ( is_array( $meta_data['image_meta'] ) ) {
							foreach( $meta_data['image_meta'] as $key => $meta_item ){
								if ( !empty( $meta_item) && $meta_item != 'unknown' ) {
									/* Perform some basic corrections to get data looking right */
									if ( $key == 'created_timestamp' ){ 
										if ( empty($attributes['output_string']) ) { 
											$key = "Date Created"; 
										} 
										$meta_item = date( "F j, Y, g:i a" , $meta_item); 
									}
									if ( $key == 'aperture' ){ $meta_item = "&fnof;/".$meta_item; }
									if ( $key == 'focal_length' ){ $meta_item = $meta_item."mm"; }
									if ( $key == 'shutter_speed' ){ 
										if ($meta_item <= 0.09999999) {
											$meta_item = "1/".(1 / $meta_item); }
										else {
											$meta_item = $meta_item * 10 . ' Seconds';
										}
									}
									/* Now output it in standard form if not $output_string is given,
									   or construct the string based on that. */
									if ( !empty($attributes['output_string']) ) {
										$attributes['output_string'] = str_replace("?".$key."?", $meta_item, $attributes['output_string']);
										$meta_list = $attributes['output_string'];
										$output_string_template = true;
									} else {
										$meta_list .= '<li>';
										$meta_list .= '<span class="meta_type">'.ucfirst(str_replace('_',' ',$key)).' - </span>'.$meta_item;
										$meta_list .= '</li>';
										$output_string_template = false;
									}
								}	
							}
						}
							//wrap in a list if using the default styles.
							$return_value = ($output_string_template) ? $meta_list : '<ul class="image_meta">' . $meta_list .'</ul>';
						
					}
				}
			} 
				
		
			//return the content. DO NOT USE ECHO.
			return "\n\n" . $this->adminOptions['display_title'] . "\n\n" . $return_value;
		}
		

    }
}

//instantiate the class
if (class_exists('fw_photo_data')) {
	$fw_photo_data = new fw_photo_data();
}




?>